from typing import Callable
from . import base as __base
from . import is_ as __is
from . import make as __mk
from . import read as __read
from . import exists_ as __exists
from . import write_ as __write
from . import path
from . import dir


def system(__command: str):
    return __base.system(__command)


def join_paths(__base, branch):
    return __base.join_paths(__base, branch)


def abspath(__name: str, is_abspath: bool = False):
    return __base.abspath(__name, is_abspath=is_abspath)


def basename(__name: str):
    return __base.basename(__name)


def parentdir(__name: str, is_abspath: bool = False):
    return __base.parentdir(__name, is_abspath)


def scandir(__name: str, is_abspath: bool, where: Callable[[object], bool] = None):
    return __base.scandir(__name, is_abspath=is_abspath, where=where)


def isfile(__name: str, is_abspath: bool = False):
    return __is.file(__name, is_abspath=is_abspath)


def isdir(__name: str, is_abspath: bool = False):
    return __is.dir(__name, is_abspath)


def exists(__name: str, is_abspath: bool = False):
    return __exists.node(__name, is_abspath)


def file_exists(__name: str, is_abspath: bool = False):
    return __exists.file(__name, is_abspath)


def dir_exists(__name: str, is_abspath: bool = False):
    return __exists.dir(__name, is_abspath)


def mkdir(__name: str, is_abspath: bool = False, exist_ok: bool = True):
    return __mk.dir(__name, is_abspath=is_abspath, exist_ok=exist_ok)


def mkfile(__name: str, is_abspath: bool = False):
    return __mk.file(__name, is_abspath)


def reads(__name: str, is_abspath: bool = False):
    return __read.string(__name, is_abspath)


def readb(__name: str, is_abspath: bool = False):
    return __read.bytes(__name, is_abspath)


def readjson(__name: str, is_abspath: bool = False):
    return __read.json(__name, is_abspath)


def readlines(
    __name: str,
    is_abspath: bool = False,
    keepends: bool = False,
    where: Callable[[str], bool] = None,
):
    return __read.lines(__name, is_abspath=is_abspath, keepends=keepends, where=where)


def writes(
    __name: str,
    content: str,
    is_abspath: bool = False,
    errors: str = "ignore",
    encoding: str = "utf-8",
    exist_ok: bool = True,
):
    return __write.string(
        __name,
        content=content,
        is_abspath=is_abspath,
        errors=errors,
        encoding=encoding,
        exist_ok=exist_ok,
    )


def writeb(
    __name: str,
    content: bytes,
    is_abspath: bool = False,
    errors: str = "ignore",
    encoding: str = "utf-8",
    exist_ok: bool = True,
):
    return __write.bytes(
        __name,
        content=content,
        is_abspath=is_abspath,
        errors=errors,
        encoding=encoding,
        exist_ok=exist_ok,
    )


def writejson(
    __name: str,
    content,
    indent: int = 4,
    is_abspath: bool = False,
    exist_ok: bool = True,
):
    return __write.json(
        __name, content=content, indent=indent, is_abspath=is_abspath, exist_ok=exist_ok
    )


def writedata(
    __name: str,
    content,
    indent: int = 4,
    is_abspath: bool = False,
    errors: str = "ignore",
    encoding: str = "utf-8",
    exist_ok: bool = True,
):
    return __write.data(
        __name,
        content=content,
        indent=indent,
        is_abspath=is_abspath,
        errors=errors,
        encoding=encoding,
        exist_ok=exist_ok,
    )


def mk_filetree(
    __ft: str, overwrite_if_exists: bool = False, parentdir: str = None, indent: int = 4
):
    return __mk.filetree(
        __ft,
        overwrite_if_exists=overwrite_if_exists,
        parentdir=parentdir,
        indent=indent,
    )


if __name__ == "__main__":
    for imported_module in (path, dir):
        if imported_module is None:
            raise ImportError(
                f"osiotk.error: failed to import submodule {imported_module.__name__}"
            )



import os as _os
from . import base as __base
from . import write_ as __write
from . import exists_ as __exists
from . import paths as _paths


def dir(__name: str, is_abspath: bool = False, exist_ok: bool = True):
    path = __base.abspath(__name, is_abspath)
    if not __exists.dir(path, is_abspath=True):
        _os.makedirs(path, exist_ok=exist_ok)


def file(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    if not __exists.file(path, True):
        __write.string(path, content="", is_abspath=True)


def filetree(
    __ft: str, overwrite_if_exists: bool = False, parentdir: str = None, indent: int = 4
):
    ft_paths = _paths.filetree(__ft, parentdir=parentdir, indent=indent)
    dirpaths = set(
        path
        for path in ft_paths
        if (
            next(
                (
                    ft_path
                    for ft_path in ft_paths
                    if (ft_path.startswith(f"{path}/") and ft_path != path)
                ),
                None,
            )
            is not None
        )
        and not "." in __base.basename(path)
    )
    for dirpath in dirpaths:
        dir(dirpath)

    for path in ft_paths:
        if not path in dirpaths:
            if __exists.file(path):
                if overwrite_if_exists:
                    __write.string(path, content="")
            else:
                file(path)

    for path in ft_paths:
        if not __exists.node(path):
            print(f"error: path does not exist:{path}")



from os import path as __ospath


def __join(__a: str, *paths):
    return __ospath.join(__a, *paths)


def __isfile(__path: str):
    return __ospath.isfile(__path)


def __created_at(__path: str):
    return __ospath.getctime(__path)


def join(__a: str, *paths):
    return __join(__a, *paths)


def isfile(__path: str):
    return __isfile(__path)


def created_at(__path: str):
    return __created_at(__path)



import os as _os
from . import base as __base


def file(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    return _os.path.isfile(path)


def dir(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    return _os.path.isdir(path)



from . import base as __base
from typing import Callable
from json import loads as _loads


def __or_none(func):
    def inner(*a, **b):
        try:
            result = func(*a, **b)
        except Exception:
            result = None
        return result

    return inner


@__or_none
def string(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    result = None
    with open(path, mode="r") as file:
        result = str(file.read())
    return result


@__or_none
def bytes(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    with open(path, mode="rb") as file:
        result = file.read()
    return result


@__or_none
def json(__name: str, is_abspath: bool = False):
    return _loads(string(__name, is_abspath=is_abspath))


@__or_none
def lines(
    __name: str,
    is_abspath: bool = False,
    keepends: bool = False,
    where: Callable[[str], bool] = None,
):
    s = string(__name, is_abspath)
    result = s.splitlines(keepends=keepends)
    if where is not None:
        result = [line for line in result if where(line)]
    return result
